﻿using JsonApiServer.Controllers;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FunctionalTests
{
   [TestClass]
   public class AuthenticateControllerTest
   {
      [TestMethod]
      public void Authenticate_succeeded()
      {
         var controller = new AuthenticateController();
         var request = new AuthenticateController.AuthenticateRequest
         {
            Username = "user8@domain.com",
            Password = "test123"
         };

         var result = controller.Authenticate(request);
         Assert.IsNotNull(result);
         Assert.IsTrue(result.Succeed);
      }

      [TestMethod]
      public void Authenticate_wrongpassword_failed()
      {
         var controller = new AuthenticateController();
         var request = new AuthenticateController.AuthenticateRequest
         {
            Username = "user8@domain.com",
            Password = "invalidpassword"
         };

         var result = controller.Authenticate(request);
         Assert.IsNotNull(result);
         Assert.IsFalse(result.Succeed);
      }

      [TestMethod]
      public void Authenticate_wrongusernamepassword_failed()
      {
         var controller = new AuthenticateController();
         var request = new AuthenticateController.AuthenticateRequest
         {
            Username = "invalidusername@domain.com",
            Password = "invalidpassword"
         };

         var result = controller.Authenticate(request);
         Assert.IsNotNull(result);
         Assert.IsFalse(result.Succeed);
      }

      [TestMethod]
      public void Authenticate_wrongusernamepassword_failed_weird_symbol()
      {
         var controller = new AuthenticateController();
         var request = new AuthenticateController.AuthenticateRequest()
         {
            Username = "\"'-=*(&%^$#@!~}{+:?>/\\invalidusername@domain.com",
            Password = "\"'-=*(&%^$#@!~}{+:?>/\\invalidpassword"
         };

         var result = controller.Authenticate(request);
         Assert.IsNotNull(result);
         Assert.IsFalse(result.Succeed);
      }
   }
}
