﻿using System;
using System.Linq;
using JsonApiServer.Controllers;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace FunctionalTests
{
   [TestClass]
   public class ReaderControllerTest
   {
      //todo: Get, GetById, GetByUsername, GetByToken, Get

      [TestMethod]
      public void GetReaders_returnFirst20Readers_sortbyDescending_succeeded()
      {
         var controller = new ReaderController();
         var filter = new ReaderController.Filter() {Contains = ""};
         var page = new ReaderController.Page() {Index = 1, Size = 20};
         var sort = new ReaderController.Sort() {Username = -1};
         var result = controller.Get(page, filter, sort);

         Assert.IsNotNull(result);
         Assert.IsTrue(result.TotalRecords > 20);
         Assert.IsTrue(
             String.Compare(result.Results.First().Username, result.Results.Last().Username, StringComparison.Ordinal) >
             0);
      }

      [TestMethod]
      public void GetReaders_returnFirst20Readers_sortbyAscending_succeeded()
      {
         var controller = new ReaderController();
         var filter = new ReaderController.Filter() { Contains = "" };
         var page = new ReaderController.Page() { Index = 1, Size = 20 };
         var sort = new ReaderController.Sort() { Username = 1 };
         var result = controller.Get(page, filter, sort);

         Assert.IsNotNull(result);
         Assert.IsTrue(result.TotalRecords > 20);
         Assert.IsTrue(
             String.Compare(result.Results.First().Username, result.Results.Last().Username, StringComparison.Ordinal) <
             0);
      }

      [TestMethod]
      public void GetReaders_returnFirst20Readers_filteredByKeyboard_succeeded()
      {
         var controller = new ReaderController();
         var filter = new ReaderController.Filter() { Contains = ".com" };
         var page = new ReaderController.Page() { Index = 1, Size = 20 };
         var sort = new ReaderController.Sort() { Username = 1 };
         var result = controller.Get(page, filter, sort);

         Assert.IsNotNull(result);
         Assert.AreEqual(result.TotalRecords, 27);
         Assert.IsTrue(result.Results.All(r => r.Username.Contains(".com")));
      }

      [TestMethod]
      public void GetAll_returnFirstFoundReaders_filteredByKeyword_succeeded()
      {
         var controller = new ReaderController();
         var filter = new ReaderController.Filter() { Contains = "user18@domain.com" };
         var page = new ReaderController.Page() { Index = 1, Size = 20 };
         var sort = new ReaderController.Sort() { Username = 1 };
         var result = controller.Get(page, filter, sort);

         Assert.IsNotNull(result);
         Assert.AreEqual(result.TotalRecords, 1);
         Assert.IsTrue(result.Results.All(r => r.Username.ToLower().Contains("user18@domain.com")));
      }

      [TestMethod]
      public void GetById_succeeded()
      {
         var controller = new ReaderController();
         string userid = "18";
         var result = controller.Get(userid);

         Assert.IsNotNull(result);
         Assert.IsNotNull(result.AccessPolicy);
         Assert.AreEqual(userid, result.Id);
         Assert.AreEqual("user18@domain.com", result.Username);
         Assert.IsTrue(result.IsActive);
      }

      [TestMethod]
      public void Get_ReaderByInvalidId_ReaderNotFound()
      {
         var controller = new ReaderController();
         string userid = "INVALID-ID-123";
         var result = controller.Get(userid);

         Assert.IsNull(result);
      }

      [TestMethod]
      public void GetByUsername_succeeded()
      {
         var controller = new ReaderController();
         string username = "user18@domain.com";

         var result = controller.GetByUserName(username);

         Assert.IsNotNull(result);
         Assert.IsNotNull(result.AccessPolicy);
         Assert.AreEqual("18", result.Id);
         Assert.AreEqual(username, result.Username);
         Assert.IsTrue(result.IsActive);
      }

      [TestMethod]
      public void GetByUsername_failed()
      {
         var controller = new ReaderController();
         string username = "invalidusername@domain.com";

         var result = controller.GetByUserName(username);

         Assert.IsNull(result);
      }

      [TestMethod]
      public void GetByToken_succeeded()
      {
         var controller = new ReaderController();

         var result = controller.GetByToken("3BBD5CC2-954A-40A1-8653-C14D8B4B4824");
         Assert.IsNotNull(result);
         Assert.AreEqual("8", result.Id);
         Assert.AreEqual("user8@domain.com", result.Username);
         Assert.IsNotNull(result.AccessPolicy);
      }

      [Ignore]
      [TestMethod]
      public void GetByToken_failed()
      {
         var controller = new ReaderController();

         var result = controller.GetByToken(Guid.Empty.ToString());
         Assert.IsNull(result);
      }
   }
}
