﻿using System;
using System.Web.Http;
using JsonApiServer.Models;
using Newtonsoft.Json;
using log4net;

namespace JsonApiServer.Controllers
{
   /// <summary>
   /// The authorization controller
   /// </summary>
   public class AuthorizeController : ApiController
   {

      private readonly ILog _log = LogManager.GetLogger(typeof(AuthorizeController));

      /// <summary>
      /// Check user's authorization based on the end user / document tuple
      /// </summary>
      /// <param name="request"></param>
      /// <returns></returns>
      [HttpPost]
      public Permission Authorize([FromBody] AuthorizeRequest request)
      {
         _log.Debug("Authorize user begins");
         _log.Debug("Authorize request: " + JsonConvert.SerializeObject(request));

         var permission = new Permission();
         
         // change this condition for your authorization
         if (CheckAuthorizationById(request.ReaderId) && CheckAuthorizationByExternalKey(request.DocExternalKey))
         {
            permission = new Permission
            {
               Succeed = true,
               AccessPolicy = null
            };
         }
         else
         {
            permission = new Permission
            {
               Succeed = false,
               //Message = "Your logged in user does not have permission to the file. Please contact your account manager for access (w29).", //uncomment if you wish to customize this message
               AccessPolicy = null
            };
         }

         _log.Debug("Result: " + JsonConvert.SerializeObject(permission));
         _log.Debug("authorize ends");

         return permission;
      }

      // for simplicity, only readers with even ids or with 'vitrium' in it are authorized
      private bool CheckAuthorizationById(string readerId)
      {
         int intId;
         if (int.TryParse(readerId, out intId))
         {
            return intId%2 == 0;
         }
         else
         {
            return readerId.Contains("vitrium");
         }
      }

      // for simplicity, the document's external key must be set to "ValidExternalKey"
      private bool CheckAuthorizationByExternalKey(string docExternalKey)
      {
         return string.Compare(docExternalKey, "ValidExternalKey", StringComparison.OrdinalIgnoreCase) == 0;
      }

      public class AuthorizeRequest
      {
         public string ReaderId { get; set; }
         public string DocId { get; set; }
         public string DocCode { get; set; }
         public string DocExternalKey { get; set; }
      }
   }
}